#!/usr/bin/env Rscript
#
# A script for generating TSV files with beta-values for a
# single pair of IDAT files. 
# 
# This script is an adaption for Reggie of "normalizeEPIC_minfi_EA_20211005.R"
# originally developed by Mattias Aine  (mattias.aine@med.lu.se), and 
# modified by Elsa Arbajian (elsa.arbajian@med.lu.se)
# 

args <- commandArgs(trailingOnly=TRUE)
idatBase <- args[1]     # Base path to a pair of IDAT files (eg. path without '_Red/Grn.idat'
outDir <- args[2]       # Path to directory where files are saved
manifestFile <- args[3] # Path to EPIC.hg38.manifest.tsv
maskFile <- args[4]     # Path to EPIC.hg38.mask.tsv
sampleName <- args[5]	# External name of Methylation items
progress <- args[6]		# Path to file for reporting progress (Reggie-style)

cat('15 Initializing R/Minfi', file=progress)
library("minfi")

# Read IDAT files
cat('20 Reading IDAT files', file=progress)
mIdat <- read.metharray(idatBase)

# Get QC
cat('30 Checking QC', file=progress)
mRaw <- preprocessRaw(mIdat)
qc <- getQC(mRaw)
detP <- detectionP(mIdat)

FLAG <- NULL
if ((qc$mMed + qc$uMed)/2 <10.5) FLAG <- 'Mean signal<10.5'
if (is.null(FLAG))
{
	if (sum(detP>0.01) / nrow(mRaw) > 0.01) FLAG <- '>1% probes with p>0.01'
}

# Noob Normalization since we want to use single-sample normalization
cat('40 Normalizing with NOOB', file=progress)
mNoob <- preprocessNoob(mIdat, dyeMethod='single')
betaData <- getBeta(mNoob)

# Load probe manifest and mask information
cat('60 Loading EPIC-850K manifest', file=progress)
hm.manifest <- read.table(manifestFile, sep='\t', header=TRUE, stringsAsFactors=FALSE)
rownames(hm.manifest) <- hm.manifest$Probe_ID
hm.mask <- read.table(maskFile, sep='\t', header=TRUE, stringsAsFactors=FALSE)
rownames(hm.mask) <- hm.mask$probeID

# Only keep data for probes that are intersecting
isect <- intersect(rownames(betaData), rownames(hm.manifest))
betaData <- betaData[isect,,drop=FALSE]
hm.mask <- hm.mask[isect,]
hm.manifest <- hm.manifest[isect,]

# Remove data for probes that have MASK_general=TRUE
keep <- ! hm.mask$MASK_general
betaData <- betaData[keep,,drop=FALSE]
hm.manifest <- hm.manifest[keep,]

cat('80 Adjusting beta-values', file=progress)

##Beta adjustment script below adapted from Markus Ringner's (Oncology & Pathology, LU) script "beta_peak_normalize.R"
##Citation: Holm et al. 2016 (Breast Cancer Res)
## ---------------------------------------------
##updated function for handling samples with non-canonical beta distributions
findMaxima <- function(x, granularity=512, bw=.02, adjust=1, from=0, to=1, kernel="epanechnikov", verbose=FALSE)
{
	aa<-density(x,kernel=kernel,n=granularity,bw=bw,adjust=adjust,from=from,to=to,na.rm=T)
	res<-vector(length=length(aa$x))
	for(i in 2:length(res[-1])) {
		res[i]<-aa$y[i]>aa$y[i-1] & aa$y[i]>aa$y[i+1]
	}
	y<-aa$x[res][order(aa$y[res],decreasing=T)] 
	if (verbose) 
	{
		cat("found peaks at:",y,"\n")
		cat("choosing min/max:",range(y),"\n") ###CHANGED here to pick out min-max peaks as these are typically the correct ones
	}
	range(y) 
}

# Separate lists for Infinium I/II probes
probes_I <- hm.manifest$type=="I"
probes_II <- hm.manifest$type=="II"

##define results matrix
meth.cal <- matrix(0, ncol=ncol(betaData), nrow=nrow(betaData))
rownames(meth.cal) <- rownames(betaData)
colnames(meth.cal) <- colnames(betaData)

for (i in 1:ncol(betaData)) 
{
	sampleId <- colnames(betaData)[i]
	
	##do pre-calibaration I vs II plot
	pdf(paste0(outDir, '/adjustmentPlots.pdf'), paper='a4', width=8, height=11, title=paste0('Beta-value adjustment: ', sampleName))
	par(mfrow=c(2,1),font=2,font.sub=2,font.lab=2,font.axis=2,las=1,mar=c(4,3,2,2))
	plot(density(betaData[probes_I,i],kernel="epanechnikov",bw=0.02,from=0,to=1,na.rm=T),main=sampleName,xlim=c(-.05,1.05),col="orange",lwd=3,xlab="pre-adjustment beta",cex.main=.9,ylim=c(0,8))
	lines(density(betaData[probes_II,i],kernel="epanechnikov",bw=0.02,from=0,to=1,na.rm=T),col="darkgreen",lwd=3)
	
	##define maxima
	tempMax_I <- findMaxima(betaData[probes_I,i])
	tempMax_II <- findMaxima(betaData[probes_II,i])
	##function not oriented so peak near zero can be in vector slot 2 -> fix
	tempMax_I <- tempMax_I[order(tempMax_I)]
	tempMax_II <- tempMax_II[order(tempMax_II)]
	##add plot "FLAG" if peaks not well separated and close to right place..
	if (is.null(FLAG))
	{
		if (tempMax_I[2] <.8 | tempMax_II[2] <.8)
		{
			FLAG <- 'Peak Meth <0.8'
		}
		else if (tempMax_I[1] >.2 | tempMax_II[1] >.2)
		{
			FLAG <- 'Peak UMeth >0.2'
		}
	}
	
	##add to plot
	abline(v=tempMax_I,col="orange",lwd=2,lty=2)
	abline(v=tempMax_II,col="darkgreen",lwd=2,lty=2)
	text(x=.5,y=c(3.5,3),labels=c("Infinium_I ","Infinium_II"),col=c("orange","darkgreen"),font=2,cex=1)
	if (!is.null(FLAG)) text(x=.5,y=c(6,5.25),labels=c("FLAGGED", FLAG),col=2,font=2,cex=1.5)
	
	##do calibration scale both to same range..
	meth.cal[probes_I,i] <- (betaData[probes_I,i]-tempMax_I[1])/(tempMax_I[2]-tempMax_I[1])
	meth.cal[probes_II,i] <- (betaData[probes_II,i]-tempMax_II[1])/(tempMax_II[2]-tempMax_II[1])
	
	##cap ends so that nothing over 1 or below zero..
	meth.cal[probes_I,i][meth.cal[probes_I,i] < 0 ] <- 0
	meth.cal[probes_I,i][meth.cal[probes_I,i] > 1 ] <- 1
	meth.cal[probes_II,i][meth.cal[probes_II,i] < 0 ] <- 0
	meth.cal[probes_II,i][meth.cal[probes_II,i] > 1 ] <- 1
	
	##do post-calibration I vs II plot
	plot(density(meth.cal[probes_I,i],kernel="epanechnikov",bw=0.02,from=0,to=1,na.rm=T),main='',xlim=c(-.05,1.05),col="orange",lwd=3,xlab="post-adjustment beta",cex.main=.9,ylim=c(0,8))
	lines(density(meth.cal[probes_II,i],kernel="epanechnikov",bw=0.02,from=0,to=1,na.rm=T),col="darkgreen",lwd=3)
	text(x=.5,y=c(3.5,3),labels=c("Infinium_I ","Infinium_II"),col=c("orange","darkgreen"),font=2,cex=1)
	abline(v=findMaxima(meth.cal[probes_I,i],verbose=F),col="orange",lwd=2,lty=2)
	abline(v=findMaxima(meth.cal[probes_II,i],verbose=F),col="darkgreen",lwd=2,lty=2)
	dev.off()
}


## End of Beta adjustment script
## -----------------------------------------------

##save and exit
cat('90 Saving beta-values to TSV', file=progress)
for (i in 1:ncol(betaData)) 
{
	##normalized beta matrix 
	write.table(betaData[,i], file=gzfile(paste0(outDir, '/beta_normalized.tsv.gz')), sep='\t', row.names=TRUE, col.names=FALSE, quote=FALSE)
	
	##adjusted beta matrix 
	write.table(meth.cal[,i], file=gzfile(paste0(outDir, '/beta_adjusted.tsv.gz')), sep='\t', row.names=TRUE, col.names=FALSE, quote=FALSE)
}

sink(paste0(outDir, '/stats.out'))
cat('MedianMeth', qc[1, 1], fill=TRUE)
cat('MedianUnmeth', qc[1, 2], fill=TRUE)
cat('ProbesRaw', nrow(mRaw), fill=TRUE)
cat('ProbesBad', sum(detP>0.01), fill=TRUE)
cat('ProbesI', sum(probes_I), fill=TRUE)
cat('ProbesII', sum(probes_II), fill=TRUE)
cat('PeakUnmethI', tempMax_I[1], fill=TRUE)
cat('PeakMethI', tempMax_I[2], fill=TRUE)
cat('PeakUnmethII', tempMax_II[1], fill=TRUE)
cat('PeakMethII', tempMax_II[2], fill=TRUE)
if (!is.null(FLAG)) cat('Flag', FLAG, fill=TRUE)
sink()

