# $Id $
#
# Useful utility functions for demuxing.
# 
# source ./demux-utils.sh

# Find the path to a run-archive folder. The result is returned to stdout.
# If the specified folder can't be found the script is exited with exit code 1.
# Parameters:
#  $1: The name of folder to look for
#  $2: List of root folders to look in (multiple folders should be separated by space)
find_run_archive()
{
	local DATA_FILES_FOLDER=$1
	local ALL_RUN_ARCHIVES=$2
	local RUN_ARCHIVE=`find ${ALL_RUN_ARCHIVES} -maxdepth 2 -name ${DATA_FILES_FOLDER} -type d -print -quit 2> /dev/null || true`
	if [ -z "$RUN_ARCHIVE" ]; then
	   echo "Can't find data folder ${DATA_FILES_FOLDER} in [${ALL_RUN_ARCHIVES}]" 1>&2
	   exit 1
	fi
	echo "${RUN_ARCHIVE}"
}

# Extract barcodes from Illumina runfolder. 
# Parameters:
#  $1: Path to Illumina folder with sequencing data
#  $2: Barcode of the flowcell
#  $3: The lane number to extract barcodes from
#  $4: Read string template
#  $5: Other options to the ExtractIlluminaBarcodes command
extract_illumina_barcodes()
{
	local RUN_ARCHIVE=$1
	local FLOWCELL_BARCODE=$2
	local LANE=$3
	local READSTRING=$4
	local MORE_OPTIONS=$5
	local PREFIX=${FLOWCELL_BARCODE}.${LANE}
	
	${WD}/stdwrap.sh ${JAVA} \
		-Dpicard.useLegacyParser=false ${JavaOptions} \
		-jar ${PICARD} ExtractIlluminaBarcodes \
		-BASECALLS_DIR ${RUN_ARCHIVE}/Data/Intensities/BaseCalls \
		-BARCODE_FILE ${WD}/${PREFIX}_barcodes.csv \
		-LANE ${LANE} \
		-READ_STRUCTURE ${READSTRING} \
		-OUTPUT_DIR ${PREFIX} \
		-METRICS_FILE ${PREFIX}_metrics.csv \
		-NUM_PROCESSORS ${NumThreads} \
		-TMP_DIR ${TMPDIR} \
		${ExtractBarcodesOptions} ${MORE_OPTIONS} \
		>> ${PREFIX}.out
}

# Extract base calls to FASTQ files.
# Parameters:
#  $1: Path to Illumina folder with sequencing data
#  $2: Barcode of the flowcell
#  $3: The lane number to extract barcodes from
#  $4: Read string template
#  $5: Other options to the IlluminaBasecallsToFastq command
basecalls_to_fastq()
{
	local RUN_ARCHIVE=$1
	local FLOWCELL_BARCODE=$2
	local LANE=$3
	local READSTRING=$4
	local MORE_OPTIONS=$5
	local PREFIX=${FLOWCELL_BARCODE}.${LANE}

	${WD}/stdwrap.sh ${JAVA} \
		-Dpicard.useLegacyParser=false ${JavaOptions} \
		-jar ${PICARD} IlluminaBasecallsToFastq \
		-BASECALLS_DIR ${RUN_ARCHIVE}/Data/Intensities/BaseCalls \
		-BARCODES_DIR ${PREFIX} \
		-MULTIPLEX_PARAMS ${WD}/${PREFIX}_multiplex.csv \
		-LANE ${LANE} \
		-FLOWCELL_BARCODE ${FLOWCELL_BARCODE} \
		-READ_STRUCTURE ${READSTRING} \
		-NUM_PROCESSORS ${NumThreads} \
		-TMP_DIR ${TMPDIR} \
		${BasecallsToFastqOptions} ${MORE_OPTIONS} \
		>> ${PREFIX}.out
}

# Merge FASTQ files to a pair for R1 and R2.
# Parameters:
#  $1: Source file name pattern
#  $2: Destination base file name.
merge_fastq_files()
{
	local BASENAME_SRC=$1
	local BASENAME_DEST=$2
	
	cat ${BASENAME_SRC}_*.1.fastq.gz > ${BASENAME_DEST}_R1.fastq.gz
	cat ${BASENAME_SRC}_*.2.fastq.gz > ${BASENAME_DEST}_R2.fastq.gz
	rm -f ${BASENAME_SRC}*
}

# Align FASTQ files with BowTie
# Parameters:
#  $1: Base file name of R1 and R2 FASTQ files
#  $2: Base file name for output files
bowtie_align()
{
	local BASENAME_SRC=$1
	local BASENAME_DEST=$2

	${WD}/stdwrap.sh ${BOWTIE} -p ${NumThreads} \
		${BowTieOptions} \
		--un-conc ${BASENAME_DEST}_R%.fastq \
		-x ${Gidx} \
		-1 ${BASENAME_SRC}_R1.fastq.gz \
		-2 ${BASENAME_SRC}_R2.fastq.gz \
		-S ${BASENAME_DEST}.sam \
		> ${BASENAME_DEST}.sam.out
	${WD}/singlecolumnaverager.awk < ${BASENAME_DEST}.sam > ${BASENAME_DEST}_fragmentsize.txt
}

# Run Trimmomatic.
# Parameters:
#  $1: Base file name for soure files
#  $2: Suffix for source files
#  $3: Base file name for destination files
#  $4: Suffix for destination files
#  $5: Other options for Trimmomatic
trimmomatic()
{
	local BASENAME_SRC=$1
	local SUFFIX_SRC=$2
	local BASENAME_DEST=$3
	local SUFFIX_DEST=$4
	local OPTIONS=$5

	$WD/stdwrap.sh ${JAVA} ${JavaOptions} \
		-jar ${TRIMMOMATIC} PE \
		-threads ${NumThreads} \
		${BASENAME_SRC}_R1.${SUFFIX_SRC} \
		${BASENAME_SRC}_R2.${SUFFIX_SRC} \
		${BASENAME_DEST}_R1.${SUFFIX_DEST} \
		${BASENAME_DEST}_R1_un.${SUFFIX_DEST} \
		${BASENAME_DEST}_R2.${SUFFIX_DEST} \
		${BASENAME_DEST}_R2_un.${SUFFIX_DEST} \
		${OPTIONS} \
		>> ${WD}/trimmomatic.out
}

# Copy a FASTQ file from the ImportArchive to given destination
# If the FASTQ file can't be found in the ImportArchive we also
# check the ImportGateway. If it is found there it is copied to
# the ImportArchive (and optionally deleted from the gateway)
# If the FASTQ can't be found an error is generated
copy_from_import_archive()
{
	local SRC=$1
	local DEST=$2
	
	if [ ! -f "${ImportArchive}/${SRC}" ]; then
		if [ ! -f "${ImportGateway}/${SRC}" ]; then
			echo "Can't find FASTQ file '${SRC}' in '${ImportGateway}' or '${ImportArchive}'" 1>&2
			exit 1
		fi
		# Copy from gatway to archive
		mkdir -p "${ImportArchive}"
		# Temporary change umask to read-only for the current user/group
		(umask u=r,g=r,o= && cp "${ImportGateway}/${SRC}" "${ImportArchive}/${SRC}")
		if [ ! "${KeepFASTQOnGateway}" ]; then
			rm -f "${ImportGateway}/${SRC}"
		fi
	fi
	if [ "${MaxFASTQLines}" ]; then
		# Debug mode to only copy part of the FASTQ file
		zcat "${ImportArchive}/${SRC}" | awk "(NR<=${MaxFASTQLines})" | gzip -1 -c > "${DEST}"
	else
		cp "${ImportArchive}/${SRC}" "${DEST}"
	fi
}
