#!/bin/bash
##
## Pipeline script for the FASTQ import step. 
## 
##
## Environment variables that should be defined (in job.sh) before calling this script
##  -AllRunArchives: White-space separated list of locations to search for sequencing data
##  -JAVA: Path to Java
##  -PICARD: Path to Picard JAR file
##  -PICARD_MEMORY: Amount of memory to assign to Java when running Picard
##  -BOWTIE: Path to BowTie program
##  -BowTieOptions: Other options for the BowTie program
##  -Gidx: Path to reference genome used for alignment with BowTie
##  -TRIMMOMATIC: Path to Trimmomatich JAR file
##  -TrimmomaticOptions1: Options for the first Trimmomatic step (adapter filter)
##  -TrimmomaticOptions2: Options for the second Trimmomatic step (quality filter)

set -e

## Import utility functions
source ./reggie-utils.sh
source ./demux-utils.sh

## Verify settings in options
rg_var_isdir "TMPDIR" "WD" "ImportGateway"
rg_var_isfile "BOWTIE" "TRIMMOMATIC" 
rg_var_isset "BowTieOptions" "Gidx" "TrimmomaticImportOptions" "TrimmomaticQualityOptions"
rg_var_isset "ImportArchive" "MergedName" "BaseFileName" "FASTQ1" "FASTQ2" "FastqFolder"

## Move to the temporary working directory and create subdirectories
cd ${TMPDIR}
mkdir fastq
mkdir fastq.aligned
mkdir fastq.trim.1
mkdir fastq.trim.2
mkdir done

# Copy FASTQ files to tmp folder
if [ ! -f "done/copy_R1.done" ]; then
	rg_progress 10 "Copying FASTQ files: ${FASTQ1}"
	copy_from_import_archive "${FASTQ1}" fastq/${BaseFileName}_R1.fastq.gz "${RemoveFASTQFromGateway}" "${MaxFASTQLines}"
	touch "done/copy_R1.done"
fi
if [ ! -f "done/copy_R2.done" ]; then
	rg_progress 15 "Copying FASTQ files: ${FASTQ2}"
	copy_from_import_archive "${FASTQ2}" fastq/${BaseFileName}_R2.fastq.gz "${RemoveFASTQFromGateway}" "${MaxFASTQLines}"
	touch "done/copy_R2.done"
fi

# Run Bowtie
if [ ! -f "done/align.done" ]; then
	rg_progress 20 "Bowtie2: ${MergedName} (${NumThreads} threads)"
	bowtie_align fastq/${BaseFileName} fastq.aligned/${BaseFileName}
	touch "done/align.done"
fi

## First Trimmomatic step is used to filter on adapter sequences
if [ ! -f "done/trim1.done" ]; then
	rg_progress 50 "Trimmomatic (adapter): ${MergedName} (${NumThreads} threads)"
	trimmomatic fastq/${BaseFileName} "fastq.gz" fastq.trim.1/${BaseFileName} "fastq" "${TrimmomaticImportOptions}"
	touch "done/trim1.done"
fi

## Second Trimmomatic step is used to filter on quality
if [ ! -f "done/trim2.done" ]; then
	rg_progress 60 "Trimmomatic (quality): ${MergedName} (${NumThreads} threads)"
	trimmomatic fastq.trim.1/${BaseFileName} "fastq" fastq.trim.2/${BaseFileName} "fastq" "${TrimmomaticQualityOptions}"
	touch "done/trim2.done"
fi

## Get statitics about average remaining read lengths
if [ ! -f "done/readlength.done" ]; then
	rg_progress 70 "Calculating average read length: ${MergedName}"
	${WD}/readlength_averager.awk < fastq.trim.2/${BaseFileName}_R1.fastq >> fastq.trim.2/${BaseFileName}_readlength.txt
	${WD}/readlength_averager.awk < fastq.trim.2/${BaseFileName}_R2.fastq >> fastq.trim.2/${BaseFileName}_readlength.txt
	touch "done/readlength.done"
fi

rg_progress 80 "Archiving FASTQ: ${MergedName}"
mkdir -p ${FastqFolder}
rm -rf ${FastqFolder}/*
cat fastq.aligned/${BaseFileName}_fragmentsize.txt >> ${WD}/fragments.out
cp fastq.aligned/${BaseFileName}_fragmentsize.txt ${FastqFolder}/${BaseFileName}_fragmentsize.txt
cat fastq.trim.2/${BaseFileName}_readlength.txt >> ${WD}/readlength.out
cp fastq.trim.2/${BaseFileName}_readlength.txt ${FastqFolder}/${BaseFileName}_readlength.txt
pigz -5 -p ${NumThreads} -c fastq.trim.2/${BaseFileName}_R1.fastq > ${FastqFolder}/${BaseFileName}_R1.fastq.gz
pigz -5 -p ${NumThreads} -c fastq.trim.2/${BaseFileName}_R2.fastq > ${FastqFolder}/${BaseFileName}_R2.fastq.gz
ls -1 ${FastqFolder}/*.fastq.gz >> ${WD}/files.out

rg_progress 99 "Analysis completed, cleaning up..."
