#!/bin/bash
##
## Pipeline script for the Legacy Tophat/Cufflinks pipeline. 
## 
##
## Environment variables that should be defined (in job.sh) before calling this script
##  -FastqFolder: Path to folder with FASTQ files
##  -CufflinksFolder: Folder for output. Will be created or cleared from existing files
##  -BOWTIE: Path to BowTie program
##  -RMidx: Path to reference genome used for masking
##  -BowTieOptions: Other options for the BowTie program (when masking)
##  -TOPHAT: Path to Tophat program
##  -ATidx: Path to the reference transcriptome used for alignment
##  -AGidx: Path to the reference genome used for alignment
##  -TophatOptions: Other options for the Tophat program
##  -JAVA: Path to java
##  -PICARD: Path to Picard JAR file
##  -MarkDuplicatesOptions: Options to the Picard MarkDuplicates step
##  -SAMTOOLS: Path to the Samtools program
##  -CUFFLINKS: Path to Cufflinks program
##  -CGTF: Path GTF file user by Cufflinks
##  -CGidx: Path to reference genome used by Cufflinks
##  -CufflinksOptions: Other options for the Cufflinks program
##  -IdRemap: Path to text file for re-mapping transcript ids
##  -TMPDIR: Path to a local temporary working directory
##

set -e

## Import utility functions
source ./reggie-utils.sh

## Verify settings in options
rg_var_isdir "FastqFolder" "TMPDIR" "WD"
rg_var_isfile "BOWTIE" "TOPHAT" "JAVA" "PICARD" "SAMTOOLS" "CUFFLINKS" "CGTF" "CGidx" "IdRemap"
rg_var_isset "RMidx" "AGidx" "ATidx" "CufflinksFolder"

## Move to the temporary working directory and create subdirectories
cd ${TMPDIR}
mkdir -p fastq
mkdir -p masked
mkdir -p aligned
mkdir -p cufflinks
mkdir -p done

## Copy FASTQ files to local working directory
if [ ! -f "done/copy.done" ]; then
	rg_progress 10 "Copying FASTQ files"
	cp ${FastqFolder}/*.fastq.gz fastq
	touch "done/copy.done"
fi

## Run Bowtie2
if [ ! -f "done/mask.done" ]; then
	rg_progress 20 "Running Bowtie2 (${NumThreads} threads)"
	## Find R1 and R2 FASTQ files
	FASTQ1=`find fastq -name "*_R1.fastq.gz" -print -quit 2> /dev/null`
	FASTQ2=`find fastq -name "*_R2.fastq.gz" -print -quit 2> /dev/null`
	rg_var_isfile "FASTQ1" "FASTQ2"
	${WD}/stdwrap.sh ${BOWTIE} \
		-p ${NumThreads} \
		${BowTieOptions} \
		--un-conc-gz masked/R%.fastq.gz \
		-x ${RMidx} \
		-1 ${FASTQ1} \
		-2 ${FASTQ2} \
		-S /dev/null \
		> masked/masked.out
	touch "done/mask.done"
fi
	
## Run Tophat 
if [ ! -f "done/align.done" ]; then
	rg_progress 30 "Running Tophat (${NumThreads} threads)"
	${WD}/stdwrap.sh ${TOPHAT} \
		-p ${NumThreads} \
		-o aligned \
		${TophatOptions} \
		--transcriptome-index ${ATidx} \
		${AGidx} \
		masked/R1.fastq.gz \
		masked/R2.fastq.gz \
		> aligned/aligned.out
	rm -rf aligned/logs
	touch "done/align.done"
fi

## MarkDuplicates
if [ ! -f "done/markduplicates.done" ]; then
	rg_progress 50 "Running picard MarkDuplicates"
	${WD}/stdwrap.sh ${JAVA} \
		-Dpicard.useLegacyParser=false ${JavaOptions} \
		-jar ${PICARD} MarkDuplicates \
		-INPUT aligned/accepted_hits.bam \
		-OUTPUT aligned/accepted_hits.bam.tmp_picard \
		-METRICS_FILE aligned/accepted_hits_picardmetrics.csv \
		${MarkDuplicatesOptions} \
		> aligned/picard_MarkDuplicates.out
	mv -f aligned/accepted_hits.bam.tmp_picard aligned/accepted_hits.bam
	touch "done/markduplicates.done"
fi

if [ ! -f "done/statistics.done" ]; then
	rg_progress 55 "Calculating alignment statistics"
	${WD}/alignment_statistics.sh aligned/accepted_hits.bam aligned/unmapped.bam > aligned/alignment_statistics.out
	${SAMTOOLS} view aligned/accepted_hits.bam | ${WD}/singlecolumnaverager.awk > aligned/fragments.out
	touch "done/statistics.done"
fi


## Cufflinks
if [ ! -f "done/cufflinks.done" ]; then
	rg_progress 60 "Running Cufflinks (${NumThreads} threads)"
	${WD}/stdwrap.sh ${CUFFLINKS} \
		-p ${NumThreads} \
		-o cufflinks \
		--GTF ${CGTF} \
		--frag-bias-correct ${CGidx} \
		${CufflinksOptions} \
		aligned/accepted_hits.bam \
		> cufflinks/cufflinks.out
	touch "done/cufflinks.done"
fi

## Remap tracking_id
if [ ! -f "done/remap.done" ]; then
	${WD}/fix_cufflinks_tracking_id.sh \
		${IdRemap} \
		< cufflinks/isoforms.fpkm_tracking \
		> cufflinks/isoforms.fpkm_tracking.tmp
	mv -f cufflinks/isoforms.fpkm_tracking.tmp cufflinks/isoforms.fpkm_tracking
	touch "done/remap.done"
fi

rg_progress 95 "Copying result files to project archive"
cp masked/*.out ${WD}
cp aligned/*.out ${WD}
cp aligned/accepted_hits_picardmetrics.csv ${WD}
cp cufflinks/*.out ${WD}

mkdir -p ${CufflinksFolder}
rm -rf ${CufflinksFolder}/*
cp cufflinks/isoforms.fpkm_tracking ${CufflinksFolder}
ls -1 ${CufflinksFolder}/* >> ${WD}/files.out

rg_progress 99 "Analysis completed, cleaning up..."

