#!/bin/bash
##
## Pipeline script for checking status of NextSeq sequencing run.
## 
## Environment variables that should be defined before calling this script
##  -AllRunArchives: White-space separated list of locations to search for sequencing data
##  -BARCODE: Barcode of the flow cell that we want to find information for
##
## The output is a number of key-value pairs. All values may not be present.
##
##  -RunArchive: The path to the data folder for the flow cell
##  -Config: Date and time the 'Config' folder was last modified
##  -RunParameters: Date and time the 'RunParameters.xml' file was last modified
##  -BgzfCount: Number of files ending with 'bgzf.bci'
##  -NumLanes: Value from <NumLanes> tag in RunParameters.xml
##  -Read1: Value from <Read1> tag in RunParameters.xml
##  -Read2: Value from <Read2> tag in RunParameters.xml
##  -Index1Read: Value from <Index1Read> tag in RunParameters.xml
##  -Index2Read: Value from <Index2Read> tag in RunParameters.xml
##  -NextSeqSerial: Value from <InstrumentID> tag in RunParameters.xml
##  -RunCompletionStatus: Date and time the 'RunCompletionStatus.xml' was last modified

# Format string for file dates/times
DATE_FORMAT="%Y%m%d %H%M%S"

# Try to find a folder inside run-archive that has the barcode in the name
# The folder may not yet exist so a missing folder is not an error
DATA_FOLDER=`find ${AllRunArchives} -maxdepth 2 -iname "*${BARCODE}*" -type d -print 2> /dev/null || true`

# Fail if more than one folder is found
readarray -t lines <<< "${DATA_FOLDER}"
if [ ! ${#lines[@]} -eq 1 ]; then
	echo "Found ${#lines[@]} data folders for flow cell ${BARCODE}" 1>&2
	echo ${DATA_FOLDER} 1>&2
	exit 1
fi

echo RunArchive: ${DATA_FOLDER}
# Config folder is created immediately when starting the NextSeq
# We use the date of this folder to set the start date of the job
if [ -d "${DATA_FOLDER}/Config" ]; then
	echo "Config: `date +"${DATE_FORMAT}" -r "${DATA_FOLDER}/Config"`"
fi

# RunParameters.xml is created after clustering
# We extract information about number of reads and lanes
# and compare that to the number of *.bgzf.bci files we can find
# This gives an estimate of the current sequencing cycle and we can
# use this for progress reporting
RUN_PARAMETERS=${DATA_FOLDER}/RunParameters.xml
if [ -f "${RUN_PARAMETERS}" ]; then
	echo "RunParameters: `date +"${DATE_FORMAT}" -r "${RUN_PARAMETERS}"`"
	echo "BgzfCount: `find "${DATA_FOLDER}" -type f -name *.bgzf.bci | wc -l`"
	echo "NumLanes: `grep '<NumLanes>' "${RUN_PARAMETERS}" | cut -d '>' -f 2 | cut -d '<' -f 1`"
	echo "Read1: `grep '<Read1>' "${RUN_PARAMETERS}" | cut -d '>' -f 2 | cut -d '<' -f 1`"
	echo "Read2: `grep '<Read2>' "${RUN_PARAMETERS}" | cut -d '>' -f 2 | cut -d '<' -f 1`"
	echo "Index1Read: `grep '<Index1Read>' "${RUN_PARAMETERS}" | cut -d '>' -f 2 | cut -d '<' -f 1`"
	echo "Index2Read: `grep '<Index2Read>' "${RUN_PARAMETERS}" | cut -d '>' -f 2 | cut -d '<' -f 1`"
	echo "NextSeqSerial: `grep '<InstrumentID>' "${RUN_PARAMETERS}" | cut -d '>' -f 2 | cut -d '<' -f 1`"
fi

# RunCompletionStatus.xml is created when everything is complete
# This becomes the end date of the job and should trigger
# Reggie to start file checks and secondary analysis
if [ -f "${DATA_FOLDER}/RunCompletionStatus.xml" ]; then
	echo "RunCompletionStatus: `date +"${DATE_FORMAT}" -r "${DATA_FOLDER}/RunCompletionStatus.xml"`"
fi
