# $Id $
#
# Utility functions for variant calling.
# 
# source ./variantcall-utils.sh

# Find number of variants in a VCF file.
# Parameters:
#  $1: The name of the VCF file
find_num_variants_in_vcf()
{
	local VCF_FILE=$1
	# The ' || true' is critical since 'grep' will exit with code 1 if there are no matches
	local NUM_VARIANTS=`grep -c -v '^#' ${VCF_FILE} || true`
	echo "${NUM_VARIANTS}"
}

# Find regions that are "callable" in the BAM file
# Parameters:
#  $1: Required depth at callable locations
#  $2: Aligned BAM file
#  $3: Output BED file with all callable regions
find_callable_regions()
{
	local MIN_DEPTH=$1
	local BAM_FILE=$2
	local BED_FILE=$3
	${MOSDEPTH} ${MosdepthOptions} \
		mosdepth/alignment \
		${BAM_FILE} \
		> mosdepth/mosdepth.out
	zcat mosdepth/alignment.per-base.bed.gz \
		| awk "\$4 >= ${MIN_DEPTH}" \
		| ${BEDTOOLS} merge \
		> ${BED_FILE}
}

# Find variants. Parameters:
#  $1: Aligned BAM file
#  $2: BED file that define regions where variants should be called
#  $3: Path to output VCF file
find_variants()
{
	local BAM_FILE=$1
	local BED_FILE=$2
	local OUT_FILE=$3
	
	# If we don't have explicit paths to teststrandbias.R
	# and var2vcf_valid.pl we assume that they are in the same
	# directory as vardict-java
	if [ ! "${VARDICT_TESTSTRANDBIAS}" ]; then
		VARDICT_TESTSTRANDBIAS="${VARDICT%/*}/teststrandbias.R"
	fi
	if [ ! "${VARDICT_VAR2VCF}" ]; then
		VARDICT_VAR2VCF="${VARDICT%/*}/var2vcf_valid.pl"
	fi
	
	${VARDICT} -th ${NumThreads} \
		-G ${Genome} \
		${VarDictOptions} \
		-b ${BAM_FILE} \
		${BED_FILE} \
		| ${VARDICT_TESTSTRANDBIAS} \
		| ${VARDICT_VAR2VCF} \
			-N ${AlignedName} \
			${Var2VcfOptions} \
		> ${OUT_FILE}
}

# Calculate GC content and DNA strength using custom script
# We need to get the reference sequence within 50 base-pairs of each entry in the VCF
# and run that through gc_stat.py which produces a VCF file that vcfanno can use
# to add back annotations to the VCF file from VarDict
# Parameters:
#  $1: Input VCF file
gc_content()
{
	local VCF_IN=$1
	awk '{OFS="\t"; if (!/^#/){print $1,($2<50)?0:$2-50,$2+50,$2,$4,$5}}' ${VCF_IN} > tmp/gc-50.bed
	${BEDTOOLS} getfasta -fi ${Genome} -bed tmp/gc-50.bed -bedOut > tmp/gc-50-fasta.bed
	cat tmp/gc-50-fasta.bed | ${WD}/gc_stat.py | bgzip -c > tmp/gc-50.vcf.gz
	tabix tmp/gc-50.vcf.gz
	# Create gc_stat.toml file so that we can annotate the VCF with GC content
	echo '[[annotation]]' > tmp/gc_stat.toml
	echo 'file="tmp/gc-50.vcf.gz"' >> tmp/gc_stat.toml
	echo 'fields=["GC_CONT","GC_CONT_LOCAL","STRENGTH","STRENGTH_LOCAL"]' >> tmp/gc_stat.toml
	echo 'names=["GC_CONT","GC_CONT_LOCAL","STRENGTH","STRENGTH_LOCAL"]' >> tmp/gc_stat.toml
	echo 'ops=["self","self","self","self"]' >> tmp/gc_stat.toml
}

# Annotate a VCF file with 'vcfanno'
# Parameters:
#  $1: Input VCF file
#  $2: Options to vcfanno
#  $3: Output VCF file
vcf_annotate()
{
	local VCF_IN=$1
	local OPTIONS=$2
	local VCF_OUT=$3
	${WD}/stderrwrap.sh ${VCFANNO} ${OPTIONS} ${VCF_IN} > ${VCF_OUT} 3>> vcfanno.out
}

# Annotate a VCF file with 'snpEff'
# Parameters:
#  $1: Input VCF file
#  $2: Options to snpEff
#  $3: Output VCF file is compressed with bgzip
snpeff_annotate()
{
	local VCF_IN=$1
	local OPTIONS=$2
	local VCF_OUT=$3
	if [ "${VCF_OUT: -3}" == ".gz" ]; then
		local COMPRESS="bgzip -c"
	fi
	${WD}/stderrwrap.sh ${JAVA} ${JavaOptions} \
		-jar ${SNPEFF} \
		${OPTIONS} ${VCF_IN} \
		3>> snpeff.out \
		| ${COMPRESS:-cat} > ${VCF_OUT}
}

# Filter a VCF file with 'SnpSift'
# Parameters:
#  $1: Input VCF file
#  $2: Options to SnpSift
#  $3: Output VCF file (not compressed)
snpsift_filter()
{
	local VCF_IN=$1
	local OPTIONS=$2
	local VCF_OUT=$3

	${WD}/stderrwrap.sh ${JAVA} ${JavaOptions} \
		-jar ${SNPSIFT} filter \
		${OPTIONS} ${VCF_IN} \
		3> snpsift.out \
		> ${VCF_OUT}
}
