#!/bin/bash
##
## Pipeline script for the VariantCall analysis. It consists of two parts:
## a raw variant calling part, and an annotation and filtering part. Two 
## environment variables control which steps that should be done:
##
##  -RawCallingVcf: Path to an existing raw variant calling VCF that should be re-used
##                  for annotating and filtering. If not set, raw variant calling is done.
##  -FilteredFolder: Folder to store the results of annotating and filtering the raw calling.
##                   If not set this step is skipped.
## 
## Parameters that are used for raw calling:
##  -BamFolder: Path to folder where alignment.bam is found (the raw VCF is saved here)
##  -BamName: Base part of alignment BAM file (eg. alignment)
##  -MOSDEPTH: Path to the mosdepth program
##  -BEDTOOLS: Path to the bedtools program
##  -VARDICT: Path to the VarDict java program
##  -VCFANNO: Path to the vcfanno program
##  -Genome: Path to the reference genome 
##  -MosdepthOptions: Options for the mosdepth progarm
##  -MinCallableDepth: Minimum coverage at a location for calling
##  -VarDictOptions: Options to the VarDict program
##  -Var2VcfOptions: Options to the VarDict step that convert results to VCF file
##
## Parameters that are used for annotating and filtering:
##  -JAVA: Path to java
##  -VCFANNO: Path to the vcfanno program
##  -SNPEFF: Path to the SnpEff JAR file
##  -SNPSIFT: Path to the SnpSift JAR file
##  -MutationSignature: Path to the RData file for mutation signature
##  -VcfannoOptions: Options to the vcfanno program
##  -SnpEffOptions: Options to the SnpSift program
##  -SnpSiftOptions: Options to the SnpSift program
##  -AlignedName: External name of the alignment
##
## Other parameters:
##  -TMPDIR: Path to a local temporary working directory
##

set -eo pipefail

## Import utility functions
source ./reggie-utils.sh
source ./variantcall-utils.sh

## Verify settings in options
rg_var_isdir "TMPDIR" "WD"
if [ "${RawCallingVcf}" ]; then
	rg_file_exists "${RawCallingVcf}"
else
	rg_var_isdir "BamFolder"
	rg_var_isfile "MOSDEPTH" "BEDTOOLS" "VARDICT" "Genome" "VCFANNO"
	rg_var_isset "BamName" "MinCallableDepth" "VarDictOptions" "Var2VcfOptions"
	rg_file_exists "${BamFolder}/${BamName}.bam" "${BamFolder}/${BamName}.bai"
fi
if [ "${FilteredFolder}" ]; then
	rg_var_isfile "VCFANNO" "SNPEFF" "SNPSIFT" "MutationSignature"
	rg_var_isset "VcfannoOptions" "SnpEffOptions" "SnpSiftOptions" "AlignedName"
fi

## Move to the temporary working directory
cd ${TMPDIR}
mkdir -p bam
mkdir -p mosdepth
mkdir -p tmp
mkdir -p resultsraw
mkdir -p resultsfilter
mkdir -p done

if [ "${RawCallingVcf}" ]; then
	# We use the existing raw calling file
	zcat ${RawCallingVcf} > tmp/variants-raw-2.vcf
else
	## Copy BAM file to local working directory
	if [ ! -f "done/copy.done" ]; then
		rg_progress 10 "Copying BAM file"
		cp ${BamFolder}/${BamName}.bam bam/alignment.bam
		cp ${BamFolder}/${BamName}.bai bam/alignment.bai
		touch "done/copy.done"
	fi

	# Find regions with depth >= 5 using mosdepth (and some help from awk)
	if [ ! -f "done/mosdepth.done" ]; then
		rg_progress 20 "Finding callable regions"
		find_callable_regions ${MinCallableDepth} bam/alignment.bam resultsraw/variants-callable.bed
		touch "done/mosdepth.done"
	fi

	if [ ! -f "done/vardict.done" ]; then
		rg_progress 30 "Running VarDict (${NumThreads} threads)"
		find_variants bam/alignment.bam resultsraw/variants-callable.bed tmp/variants-raw-1.vcf
		touch "done/vardict.done"
	fi

	if [ ! -f "done/gccontent.done" ]; then
		NUMVARIANTS=`find_num_variants_in_vcf tmp/variants-raw-1.vcf`
		if [ ${NUMVARIANTS} -gt 0 ]; then
			gc_content tmp/variants-raw-1.vcf
			vcf_annotate tmp/variants-raw-1.vcf tmp/gc_stat.toml tmp/variants-raw-2.vcf
		else
			# No variants found, simply copy the raw file
			cp tmp/variants-raw-1.vcf tmp/variants-raw-2.vcf
		fi
		touch "done/gccontent.done"
	fi
	
	if [ ! -f "done/rawcalling.done" ]; then
		# We now have the raw variant calling file
		cat tmp/variants-raw-2.vcf | bgzip -c > resultsraw/variants-raw.vcf.gz
		touch "done/rawcalling.done"
	fi
fi

if [ "${FilteredFolder}" ]; then
	# We should annotate and filter the raw calling
	NUMVARIANTS=`find_num_variants_in_vcf tmp/variants-raw-2.vcf`
	if [ $NUMVARIANTS -gt 0 ]; then
		# Annotate variants in two steps, first with vcfanno and several databases
		if [ ! -f "done/vcfanno.done" ]; then
			rg_progress 60 "Annotating variants (vcfanno; step 1 of 2)"
			vcf_annotate tmp/variants-raw-2.vcf "${VcfannoOptions}" tmp/variants-annotated-2.vcf
			touch "done/vcfanno.done"
		fi
		
		# Second annotation step with snpEff
		if [ ! -f "done/snpeff.done" ]; then
			rg_progress 70 "Annotating variants (snpEff; step 2 of 2)"
			snpeff_annotate tmp/variants-annotated-2.vcf "${SnpEffOptions}" resultsfilter/variants-annotated.vcf.gz
			touch "done/snpeff.done"
		fi
		
		if [ ! -f "done/snpsift.done" ]; then
			rg_progress 80 "Filtering variants (SnpSift)"
			snpsift_filter resultsfilter/variants-annotated.vcf.gz "${SnpSiftOptions}" resultsfilter/variants-filtered.vcf
			touch "done/snpsift.done"
		fi
		
		if [ ! -f "done/mutsign.done" ]; then
			NUMFILTERED=`find_num_variants_in_vcf resultsfilter/variants-filtered.vcf`
			if [ $NUMFILTERED -gt 0 ]; then
				${RSCRIPT} ${WD}/mutation_signature.R ${AlignedName} \
					resultsfilter/variants-filtered.vcf \
					${MutationSignature} \
					resultsfilter/mutation_signature.pdf \
					> resultsfilter/mutation_signature.txt
			fi
			touch "done/mutsign.done"
		fi
	else
		# No variants in the raw VCF file -- we make a copies of it 'variants-annotated.vcf.gz' and 'variants-filtered.vcf'
		cat tmp/variants-raw-2.vcf | bgzip -c > resultsfilter/variants-annotated.vcf.gz
		cp tmp/variants-raw-2.vcf resultsfilter/variants-filtered.vcf
	fi
fi

rg_progress 90 "Copying result files to project archive"
if [ ! "${RawCallingVcf}" ]; then
	\cp resultsraw/variants-* ${BamFolder}
	ls -1 resultsraw/variants-* >> ${WD}/files.out
	echo "Callable bases: `awk -F'\\t' 'BEGIN{SUM=0}{ SUM+=$3-$2 } END{print SUM}' resultsraw/variants-callable.bed`" >> ${WD}/stats.out
	echo "Raw variants: `zcat resultsraw/variants-raw.vcf.gz | grep -c -v '^#'`" >> ${WD}/stats.out
fi
if [ "${FilteredFolder}" ]; then
	mkdir -p ${FilteredFolder}
	rm -rf ${FilteredFolder}/*
	cp resultsfilter/* ${FilteredFolder}
	ls -1 ${FilteredFolder}/* >> ${WD}/files.out
	echo "Annotated variants: `zcat resultsfilter/variants-annotated.vcf.gz | grep -c -v '^#'`" >> ${WD}/stats.out
	echo "Filtered variants: `cat resultsfilter/variants-filtered.vcf | grep -c -v '^#'`" >> ${WD}/stats.out
fi

rg_progress 99 "Analysis completed, cleaning up..."
